from NLPyPort.LemPyPort.normalization.verb_normalizer import *
from NLPyPort.LemPyPort.normalization.superlative_normalizer import *
from NLPyPort.LemPyPort.normalization.number_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_name_normalizer import *
from NLPyPort.LemPyPort.normalization.diminutive_normalizer import *
from NLPyPort.LemPyPort.normalization.augmentative_normalizer import *
from NLPyPort.LemPyPort.normalization.adverb_normalizer import *
